package net.knarcraft.permissionsigns;

import net.knarcraft.knarlib.formatting.StringFormatter;
import net.knarcraft.knarlib.formatting.Translator;
import net.knarcraft.knarlib.util.UpdateChecker;
import net.knarcraft.permissionsigns.command.PermissionSignsCommand;
import net.knarcraft.permissionsigns.command.PermissionSignsTabCompleter;
import net.knarcraft.permissionsigns.config.Configuration;
import net.knarcraft.permissionsigns.config.Translatable;
import net.knarcraft.permissionsigns.listener.BlockListener;
import net.knarcraft.permissionsigns.listener.PlayerListener;
import net.knarcraft.permissionsigns.listener.SignListener;
import net.knarcraft.permissionsigns.manager.EconomyManager;
import net.knarcraft.permissionsigns.manager.PermissionManager;
import net.knarcraft.permissionsigns.manager.SignCreationManager;
import net.knarcraft.permissionsigns.manager.SignManager;
import net.knarcraft.permissionsigns.thread.PermissionTimeoutThread;
import net.milkbowl.vault.economy.Economy;
import net.milkbowl.vault.permission.Permission;
import org.bukkit.Bukkit;
import org.bukkit.command.PluginCommand;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.RegisteredServiceProvider;
import org.bukkit.plugin.ServicesManager;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.scheduler.BukkitScheduler;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;

/*
PermissionSigns - A permission shop plugin for Spigot
Copyright (C) 2022 Kristian Knarvik (EpicKnarvik97)

The following license notice applies to all source and resource files in the PermissionSigns project:

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The main permissionsigns class
 */
public final class PermissionSigns extends JavaPlugin {

    private static PermissionSigns instance;
    private Translator translator;
    private StringFormatter stringFormatter;
    private Configuration configuration;

    /**
     * Instantiates the permission signs class
     */
    @SuppressWarnings("unused")
    public PermissionSigns() {
        super();
        instance = this;
    }

    /**
     * Gets an instance of this plugin
     *
     * @return <p>An instance of this plugin</p>
     */
    public static PermissionSigns getInstance() {
        return instance;
    }

    /**
     * Gets the translator
     *
     * @return <p>The translator</p>
     */
    public @NotNull Translator getTranslator() {
        return translator;
    }

    /**
     * Gets the string formatter
     *
     * @return <p>The string formatter</p>
     */
    public @NotNull StringFormatter stringFormatter() {
        return stringFormatter;
    }

    /**
     * Gets this plugin's configuration
     *
     * @return <p>This plugin's configuration</p>
     */
    public Configuration getConfiguration() {
        return configuration;
    }

    /**
     * Gets the version of this plugin
     *
     * @return <p>This plugin's version</p>
     */
    public String getPluginVersion() {
        return this.getDescription().getVersion();
    }

    @Override
    public void reloadConfig() {
        super.reloadConfig();
        loadConfig();
    }

    @Override
    public void onEnable() {
        //Check if vault is loaded
        setupVault();

        //Load config and write the default config if necessary
        FileConfiguration config = this.getConfig();
        this.saveDefaultConfig();
        config.options().copyDefaults(true);

        translator = new Translator();
        translator.registerMessageCategory(Translatable.ALREADY_HAS_PERMISSIONS);
        stringFormatter = new StringFormatter(this.getDescription().getName(), translator);
        configuration = new Configuration(this.getConfig(), translator, this.getDataFolder());

        registerListeners();
        registerCommands();
        runThreads();

        SignManager.loadSigns();
        PermissionManager.loadTemporaryPermissions();

        UpdateChecker.checkForUpdate(this, "https://api.spigotmc.org/legacy/update.php?resource=99426",
                () -> this.getDescription().getVersion(), null);
    }

    @Override
    public void onDisable() {
        try {
            SignManager.saveSigns();
        } catch (IOException e) {
            e.printStackTrace();
        }
        SignCreationManager.stopTimeoutTimer();
    }

    /**
     * Loads the config file
     */
    private void loadConfig() {
        this.reloadConfig();
        configuration.loadConfig(this.getConfig());
        saveConfig();
    }

    /**
     * Starts all separate threads for executing tasks
     */
    private void runThreads() {
        BukkitScheduler scheduler = Bukkit.getScheduler();
        SignCreationManager.runTimeoutTimer();
        scheduler.runTaskTimer(this, new PermissionTimeoutThread(), 0L, 25L);
    }

    /**
     * Registers all necessary listeners
     */
    private void registerListeners() {
        PluginManager pluginManager = getServer().getPluginManager();
        pluginManager.registerEvents(new SignListener(), this);
        pluginManager.registerEvents(new PlayerListener(), this);
        pluginManager.registerEvents(new BlockListener(), this);
    }

    /**
     * Sets up Vault by getting plugins from their providers
     */
    private void setupVault() {
        ServicesManager servicesManager = this.getServer().getServicesManager();
        RegisteredServiceProvider<Permission> permissionProvider = servicesManager.getRegistration(Permission.class);
        RegisteredServiceProvider<Economy> economyProvider = servicesManager.getRegistration(Economy.class);
        if (permissionProvider != null && economyProvider != null) {
            EconomyManager.initialize(economyProvider.getProvider());
            PermissionManager.initialize(permissionProvider.getProvider());
        } else {
            throw new IllegalStateException("[PermissionSigns] Error: Vault could not be loaded");
        }
    }

    /**
     * Registers a command for this plugin
     */
    private void registerCommands() {
        PluginCommand permissionSignsCommand = this.getCommand("permissionsigns");
        if (permissionSignsCommand != null) {
            permissionSignsCommand.setExecutor(new PermissionSignsCommand());
            permissionSignsCommand.setTabCompleter(new PermissionSignsTabCompleter());
        }
    }

}
