package net.knarcraft.permissionsigns.manager;

import net.knarcraft.permissionsigns.PermissionSigns;
import net.knarcraft.permissionsigns.container.PermissionSign;
import net.knarcraft.permissionsigns.container.SignCreationRequest;
import net.knarcraft.permissionsigns.thread.SignCreationRequestTimeoutThread;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitTask;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.PriorityQueue;
import java.util.Queue;
import java.util.UUID;
import java.util.stream.Stream;

/**
 * A manager for keeping track of sign creation requests
 */
public final class SignCreationManager {

    private static final Queue<SignCreationRequest> signCreationRequests = new PriorityQueue<>();
    private static BukkitTask timeoutTask = null;

    private SignCreationManager() {

    }

    /**
     * Runs the task for timing out permission sign creation requests
     */
    public static void runTimeoutTimer() {
        timeoutTask = Bukkit.getScheduler().runTaskTimer(PermissionSigns.getInstance(),
                new SignCreationRequestTimeoutThread(signCreationRequests), 0L, 1000L);
    }

    /**
     * Stops the task for timing out permission sign creation requests
     */
    public static void stopTimeoutTimer() {
        if (timeoutTask != null) {
            timeoutTask.cancel();
        }
    }

    /**
     * Adds a new sign creation request
     *
     * @param player <p>The player that initiated the sign creation</p>
     * @param sign   <p>The sign the player is about to create</p>
     */
    public static void addSignCreationRequest(@NotNull Player player, @NotNull PermissionSign sign) {
        signCreationRequests.add(new SignCreationRequest(sign, player, System.currentTimeMillis()));
    }

    /**
     * Gets the sign creation request for the player with the given UUID
     *
     * @param uuid <p>The UUID to get a sign creation request for</p>
     * @return <p>A sign creation request, or null if the UUID is not found</p>
     */
    public static @Nullable SignCreationRequest getSignCreationRequest(@NotNull UUID uuid) {
        Stream<SignCreationRequest> matchingRequests = signCreationRequests.stream().filter(
                (item) -> item.getPlayer().getUniqueId().equals(uuid));
        List<SignCreationRequest> requestList = matchingRequests.toList();
        if (!requestList.isEmpty()) {
            return requestList.get(0);
        } else {
            return null;
        }
    }

    /**
     * Cancels the sign creation request triggered by the user
     *
     * @param uuid <p>The UUID of the player to cancel the request for</p>
     */
    public static void cancelSignCreationRequest(UUID uuid) {
        Stream<SignCreationRequest> matchingRequests = signCreationRequests.stream().filter(
                (item) -> item.getPlayer().getUniqueId().equals(uuid));
        List<SignCreationRequest> requestList = matchingRequests.toList();
        if (requestList.size() > 0) {
            signCreationRequests.remove(requestList.get(0));
        }
    }

}
